@@ -1,26 +1,30 @@
 """Top-level package for PyGeoUtils."""
+from ._utils import transform2tuple
 from .exceptions import (
     EmptyResponse,
     InvalidInputRange,
     InvalidInputType,
     InvalidInputValue,
     MissingAttribute,
+    MissingColumns,
     MissingCRS,
+    UnprojectedCRS,
 )
 from .print_versions import show_versions
 from .pygeoutils import (
     Coordinates,
     GeoBSpline,
     arcgis2geojson,
+    break_lines,
     geo2polygon,
     get_gtiff_attrs,
     get_transform,
     gtiff2xarray,
     json2geodf,
+    snap2nearest,
     xarray2geodf,
     xarray_geomask,
 )
-from .utils import transform2tuple
 
 try:
     import importlib.metadata as metadata
@@ -34,10 +38,12 @@ except Exception:
 
 __all__ = [
     "arcgis2geojson",
+    "break_lines",
     "geo2polygon",
     "get_gtiff_attrs",
     "get_transform",
     "gtiff2xarray",
+    "snap2nearest",
     "xarray2geodf",
     "json2geodf",
     "transform2tuple",
@@ -48,7 +54,9 @@ __all__ = [
     "InvalidInputValue",
     "InvalidInputRange",
     "MissingAttribute",
+    "MissingColumns",
     "MissingCRS",
+    "UnprojectedCRS",
     "EmptyResponse",
     "show_versions",
 ]
