@@ -2,6 +2,23 @@
 from typing import Generator, List, Optional, Union
 
 
+class MissingColumns(Exception):
+    """Exception raised when a required column is missing from a dataframe.
+
+    Parameters
+    ----------
+    missing : list
+        List of missing columns.
+    """
+
+    def __init__(self, missing: List[str]) -> None:
+        self.message = "The following columns are missing:\n" + f"{', '.join(missing)}"
+        super().__init__(self.message)
+
+    def __str__(self) -> str:
+        return self.message
+
+
 class MissingCRS(Exception):
     """Exception raised when CRS is not given."""
 
@@ -13,6 +30,17 @@ class MissingCRS(Exception):
         return self.message
 
 
+class UnprojectedCRS(Exception):
+    """Exception raised when CRS is not given."""
+
+    def __init__(self) -> None:
+        self.message = "CRS must be projected."
+        super().__init__(self.message)
+
+    def __str__(self) -> str:
+        return self.message
+
+
 class EmptyResponse(Exception):
     """Exception raised when the input response is empty."""
 
